@@ -2,8 +2,8 @@
 
 __author__ = 'Fabio Caccamo'
 __copyright__ = 'Copyright (c) 2019 Fabio Caccamo'
-__description__ = 'The Python dictionary for humans dealing with evil/complex data.'
+__description__ = 'python-benedict is a dict subclass with keypath support, I/O shortcuts (json, toml, xml, yaml) and many utilities... for humans, obviously.'
 __email__ = 'fabio.caccamo@gmail.com'
 __license__ = 'MIT'
 __title__ = 'benedict'
-__version__ = '0.9.0'
+__version__ = '0.10.0'
